@@ -9,12 +9,12 @@ import (
 	"path/filepath"
 	"syscall"
 
-	"github.com/restic/restic/internal/errors"
-	"github.com/restic/restic/internal/restic"
-
 	"github.com/restic/restic/internal/backend"
+	"github.com/restic/restic/internal/backend/sema"
 	"github.com/restic/restic/internal/debug"
+	"github.com/restic/restic/internal/errors"
 	"github.com/restic/restic/internal/fs"
+	"github.com/restic/restic/internal/restic"
 
 	"github.com/cenkalti/backoff/v4"
 )
@@ -22,7 +22,9 @@ import (
 // Local is a backend in a local directory.
 type Local struct {
 	Config
+	sem sema.Semaphore
 	backend.Layout
+	backend.Modes
 }
 
 // ensure statically that *Local implements restic.Backend.
@@ -30,15 +32,33 @@ var _ restic.Backend = &Local{}
 
 const defaultLayout = "default"
 
-// Open opens the local backend as specified by config.
-func Open(ctx context.Context, cfg Config) (*Local, error) {
-	debug.Log("open local backend at %v (layout %q)", cfg.Path, cfg.Layout)
+func open(ctx context.Context, cfg Config) (*Local, error) {
 	l, err := backend.ParseLayout(ctx, &backend.LocalFilesystem{}, cfg.Layout, defaultLayout, cfg.Path)
 	if err != nil {
 		return nil, err
 	}
 
-	return &Local{Config: cfg, Layout: l}, nil
+	sem, err := sema.New(cfg.Connections)
+	if err != nil {
+		return nil, err
+	}
+
+	fi, err := fs.Stat(l.Filename(restic.Handle{Type: restic.ConfigFile}))
+	m := backend.DeriveModesFromFileInfo(fi, err)
+	debug.Log("using (%03O file, %03O dir) permissions", m.File, m.Dir)
+
+	return &Local{
+		Config: cfg,
+		Layout: l,
+		sem:    sem,
+		Modes:  m,
+	}, nil
+}
+
+// Open opens the local backend as specified by config.
+func Open(ctx context.Context, cfg Config) (*Local, error) {
+	debug.Log("open local backend at %v (layout %q)", cfg.Path, cfg.Layout)
+	return open(ctx, cfg)
 }
 
 // Create creates all the necessary files and directories for a new local
@@ -46,16 +66,11 @@ func Open(ctx context.Context, cfg Config) (*Local, error) {
 func Create(ctx context.Context, cfg Config) (*Local, error) {
 	debug.Log("create local backend at %v (layout %q)", cfg.Path, cfg.Layout)
 
-	l, err := backend.ParseLayout(ctx, &backend.LocalFilesystem{}, cfg.Layout, defaultLayout, cfg.Path)
+	be, err := open(ctx, cfg)
 	if err != nil {
 		return nil, err
 	}
 
-	be := &Local{
-		Config: cfg,
-		Layout: l,
-	}
-
 	// test if config file already exists
 	_, err = fs.Lstat(be.Filename(restic.Handle{Type: restic.ConfigFile}))
 	if err == nil {
@@ -64,7 +79,7 @@ func Create(ctx context.Context, cfg Config) (*Local, error) {
 
 	// create paths for data and refs
 	for _, d := range be.Paths() {
-		err := fs.MkdirAll(d, backend.Modes.Dir)
+		err := fs.MkdirAll(d, be.Modes.Dir)
 		if err != nil {
 			return nil, errors.WithStack(err)
 		}
@@ -73,6 +88,10 @@ func Create(ctx context.Context, cfg Config) (*Local, error) {
 	return be, nil
 }
 
+func (b *Local) Connections() uint {
+	return b.Config.Connections
+}
+
 // Location returns this backend's location (the directory name).
 func (b *Local) Location() string {
 	return b.Path
@@ -83,6 +102,11 @@ func (b *Local) Hasher() hash.Hash {
 	return nil
 }
 
+// HasAtomicReplace returns whether Save() can atomically replace files
+func (b *Local) HasAtomicReplace() bool {
+	return true
+}
+
 // IsNotExist returns true if the error is caused by a non existing file.
 func (b *Local) IsNotExist(err error) bool {
 	return errors.Is(err, os.ErrNotExist)
@@ -105,6 +129,9 @@ func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReade
 		}
 	}()
 
+	b.sem.GetToken()
+	defer b.sem.ReleaseToken()
+
 	// Create new file with a temporary name.
 	tmpname := filepath.Base(finalname) + "-tmp-"
 	f, err := tempFile(dir, tmpname)
@@ -113,7 +140,7 @@ func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReade
 		debug.Log("error %v: creating dir", err)
 
 		// error is caused by a missing directory, try to create it
-		mkdirErr := fs.MkdirAll(dir, backend.Modes.Dir)
+		mkdirErr := fs.MkdirAll(dir, b.Modes.Dir)
 		if mkdirErr != nil {
 			debug.Log("error creating dir %v: %v", dir, mkdirErr)
 		} else {
@@ -173,7 +200,7 @@ func (b *Local) Save(ctx context.Context, h restic.Handle, rd restic.RewindReade
 	// try to mark file as read-only to avoid accidential modifications
 	// ignore if the operation fails as some filesystems don't allow the chmod call
 	// e.g. exfat and network file systems with certain mount options
-	err = setFileReadonly(finalname, backend.Modes.File)
+	err = setFileReadonly(finalname, b.Modes.File)
 	if err != nil && !os.IsPermission(err) {
 		return errors.WithStack(err)
 	}
@@ -199,24 +226,29 @@ func (b *Local) openReader(ctx context.Context, h restic.Handle, length int, off
 		return nil, errors.New("offset is negative")
 	}
 
+	b.sem.GetToken()
 	f, err := fs.Open(b.Filename(h))
 	if err != nil {
+		b.sem.ReleaseToken()
 		return nil, err
 	}
 
 	if offset > 0 {
 		_, err = f.Seek(offset, 0)
 		if err != nil {
+			b.sem.ReleaseToken()
 			_ = f.Close()
 			return nil, err
 		}
 	}
 
+	r := b.sem.ReleaseTokenOnClose(f, nil)
+
 	if length > 0 {
-		return backend.LimitReadCloser(f, int64(length)), nil
+		return backend.LimitReadCloser(r, int64(length)), nil
 	}
 
-	return f, nil
+	return r, nil
 }
 
 // Stat returns information about a blob.
@@ -226,6 +258,9 @@ func (b *Local) Stat(ctx context.Context, h restic.Handle) (restic.FileInfo, err
 		return restic.FileInfo{}, backoff.Permanent(err)
 	}
 
+	b.sem.GetToken()
+	defer b.sem.ReleaseToken()
+
 	fi, err := fs.Stat(b.Filename(h))
 	if err != nil {
 		return restic.FileInfo{}, errors.WithStack(err)
@@ -237,6 +272,10 @@ func (b *Local) Stat(ctx context.Context, h restic.Handle) (restic.FileInfo, err
 // Test returns true if a blob of the given type and name exists in the backend.
 func (b *Local) Test(ctx context.Context, h restic.Handle) (bool, error) {
 	debug.Log("Test %v", h)
+
+	b.sem.GetToken()
+	defer b.sem.ReleaseToken()
+
 	_, err := fs.Stat(b.Filename(h))
 	if err != nil {
 		if b.IsNotExist(err) {
@@ -253,6 +292,9 @@ func (b *Local) Remove(ctx context.Context, h restic.Handle) error {
 	debug.Log("Remove %v", h)
 	fn := b.Filename(h)
 
+	b.sem.GetToken()
+	defer b.sem.ReleaseToken()
+
 	// reset read-only flag
 	err := fs.Chmod(fn, 0666)
 	if err != nil && !os.IsPermission(err) {
