@@ -11,6 +11,7 @@ import (
 	"sync"
 
 	"github.com/restic/restic/internal/backend"
+	"github.com/restic/restic/internal/backend/sema"
 	"github.com/restic/restic/internal/debug"
 	"github.com/restic/restic/internal/errors"
 	"github.com/restic/restic/internal/restic"
@@ -25,17 +26,26 @@ var _ restic.Backend = &MemoryBackend{}
 
 var errNotFound = errors.New("not found")
 
+const connectionCount = 2
+
 // MemoryBackend is a mock backend that uses a map for storing all data in
 // memory. This should only be used for tests.
 type MemoryBackend struct {
 	data memMap
 	m    sync.Mutex
+	sem  sema.Semaphore
 }
 
 // New returns a new backend that saves all data in a map in memory.
 func New() *MemoryBackend {
+	sem, err := sema.New(connectionCount)
+	if err != nil {
+		panic(err)
+	}
+
 	be := &MemoryBackend{
 		data: make(memMap),
+		sem:  sem,
 	}
 
 	debug.Log("created new memory backend")
@@ -45,6 +55,9 @@ func New() *MemoryBackend {
 
 // Test returns whether a file exists.
 func (be *MemoryBackend) Test(ctx context.Context, h restic.Handle) (bool, error) {
+	be.sem.GetToken()
+	defer be.sem.ReleaseToken()
+
 	be.m.Lock()
 	defer be.m.Unlock()
 
@@ -59,7 +72,7 @@ func (be *MemoryBackend) Test(ctx context.Context, h restic.Handle) (bool, error
 
 // IsNotExist returns true if the file does not exist.
 func (be *MemoryBackend) IsNotExist(err error) bool {
-	return errors.Cause(err) == errNotFound
+	return errors.Is(err, errNotFound)
 }
 
 // Save adds new Data to the backend.
@@ -68,6 +81,9 @@ func (be *MemoryBackend) Save(ctx context.Context, h restic.Handle, rd restic.Re
 		return backoff.Permanent(err)
 	}
 
+	be.sem.GetToken()
+	defer be.sem.ReleaseToken()
+
 	be.m.Lock()
 	defer be.m.Unlock()
 
@@ -120,6 +136,7 @@ func (be *MemoryBackend) openReader(ctx context.Context, h restic.Handle, length
 		return nil, backoff.Permanent(err)
 	}
 
+	be.sem.GetToken()
 	be.m.Lock()
 	defer be.m.Unlock()
 
@@ -131,15 +148,18 @@ func (be *MemoryBackend) openReader(ctx context.Context, h restic.Handle, length
 	debug.Log("Load %v offset %v len %v", h, offset, length)
 
 	if offset < 0 {
+		be.sem.ReleaseToken()
 		return nil, errors.New("offset is negative")
 	}
 
 	if _, ok := be.data[h]; !ok {
+		be.sem.ReleaseToken()
 		return nil, errNotFound
 	}
 
 	buf := be.data[h]
 	if offset > int64(len(buf)) {
+		be.sem.ReleaseToken()
 		return nil, errors.New("offset beyond end of file")
 	}
 
@@ -148,18 +168,21 @@ func (be *MemoryBackend) openReader(ctx context.Context, h restic.Handle, length
 		buf = buf[:length]
 	}
 
-	return ioutil.NopCloser(bytes.NewReader(buf)), ctx.Err()
+	return be.sem.ReleaseTokenOnClose(ioutil.NopCloser(bytes.NewReader(buf)), nil), ctx.Err()
 }
 
 // Stat returns information about a file in the backend.
 func (be *MemoryBackend) Stat(ctx context.Context, h restic.Handle) (restic.FileInfo, error) {
-	be.m.Lock()
-	defer be.m.Unlock()
-
 	if err := h.Valid(); err != nil {
 		return restic.FileInfo{}, backoff.Permanent(err)
 	}
 
+	be.sem.GetToken()
+	defer be.sem.ReleaseToken()
+
+	be.m.Lock()
+	defer be.m.Unlock()
+
 	h.ContainedBlobType = restic.InvalidBlob
 	if h.Type == restic.ConfigFile {
 		h.Name = ""
@@ -177,6 +200,9 @@ func (be *MemoryBackend) Stat(ctx context.Context, h restic.Handle) (restic.File
 
 // Remove deletes a file from the backend.
 func (be *MemoryBackend) Remove(ctx context.Context, h restic.Handle) error {
+	be.sem.GetToken()
+	defer be.sem.ReleaseToken()
+
 	be.m.Lock()
 	defer be.m.Unlock()
 
@@ -229,6 +255,10 @@ func (be *MemoryBackend) List(ctx context.Context, t restic.FileType, fn func(re
 	return ctx.Err()
 }
 
+func (be *MemoryBackend) Connections() uint {
+	return connectionCount
+}
+
 // Location returns the location of the backend (RAM).
 func (be *MemoryBackend) Location() string {
 	return "RAM"
@@ -239,6 +269,11 @@ func (be *MemoryBackend) Hasher() hash.Hash {
 	return md5.New()
 }
 
+// HasAtomicReplace returns whether Save() can atomically replace files
+func (be *MemoryBackend) HasAtomicReplace() bool {
+	return false
+}
+
 // Delete removes all data in the backend.
 func (be *MemoryBackend) Delete(ctx context.Context) error {
 	be.m.Lock()
