@@ -18,12 +18,16 @@ func TestRcloneExit(t *testing.T) {
 	cfg := NewConfig()
 	cfg.Remote = dir
 	be, err := Open(cfg, nil)
-	if e, ok := errors.Cause(err).(*exec.Error); ok && e.Err == exec.ErrNotFound {
+	var e *exec.Error
+	if errors.As(err, &e) && e.Err == exec.ErrNotFound {
 		t.Skipf("program %q not found", e.Name)
 		return
 	}
 	rtest.OK(t, err)
-	defer be.Close()
+	defer func() {
+		// ignore the error as the test will kill rclone (see below)
+		_ = be.Close()
+	}()
 
 	err = be.cmd.Process.Kill()
 	rtest.OK(t, err)
