@@ -17,6 +17,7 @@ import (
 	"golang.org/x/net/context/ctxhttp"
 
 	"github.com/restic/restic/internal/backend"
+	"github.com/restic/restic/internal/backend/sema"
 	"github.com/restic/restic/internal/debug"
 	"github.com/restic/restic/internal/errors"
 	"github.com/restic/restic/internal/restic"
@@ -29,9 +30,10 @@ var _ restic.Backend = &Backend{}
 
 // Backend uses the REST protocol to access data stored on a server.
 type Backend struct {
-	url    *url.URL
-	sem    *backend.Semaphore
-	client *http.Client
+	url         *url.URL
+	connections uint
+	sem         sema.Semaphore
+	client      *http.Client
 	backend.Layout
 }
 
@@ -45,7 +47,7 @@ const (
 func Open(cfg Config, rt http.RoundTripper) (*Backend, error) {
 	client := &http.Client{Transport: rt}
 
-	sem, err := backend.NewSemaphore(cfg.Connections)
+	sem, err := sema.New(cfg.Connections)
 	if err != nil {
 		return nil, err
 	}
@@ -57,10 +59,11 @@ func Open(cfg Config, rt http.RoundTripper) (*Backend, error) {
 	}
 
 	be := &Backend{
-		url:    cfg.URL,
-		client: client,
-		Layout: &backend.RESTLayout{URL: url, Join: path.Join},
-		sem:    sem,
+		url:         cfg.URL,
+		client:      client,
+		Layout:      &backend.RESTLayout{URL: url, Join: path.Join},
+		connections: cfg.Connections,
+		sem:         sem,
 	}
 
 	return be, nil
@@ -105,6 +108,10 @@ func Create(ctx context.Context, cfg Config, rt http.RoundTripper) (*Backend, er
 	return be, nil
 }
 
+func (b *Backend) Connections() uint {
+	return b.connections
+}
+
 // Location returns this backend's location (the server's URL).
 func (b *Backend) Location() string {
 	return b.url.String()
@@ -115,6 +122,12 @@ func (b *Backend) Hasher() hash.Hash {
 	return nil
 }
 
+// HasAtomicReplace returns whether Save() can atomically replace files
+func (b *Backend) HasAtomicReplace() bool {
+	// rest-server prevents overwriting
+	return false
+}
+
 // Save stores data in the backend at the handle.
 func (b *Backend) Save(ctx context.Context, h restic.Handle, rd restic.RewindReader) error {
 	if err := h.Valid(); err != nil {
@@ -157,21 +170,20 @@ func (b *Backend) Save(ctx context.Context, h restic.Handle, rd restic.RewindRea
 	return errors.Wrap(cerr, "Close")
 }
 
-// ErrIsNotExist is returned whenever the requested file does not exist on the
+// notExistError is returned whenever the requested file does not exist on the
 // server.
-type ErrIsNotExist struct {
+type notExistError struct {
 	restic.Handle
 }
 
-func (e ErrIsNotExist) Error() string {
+func (e *notExistError) Error() string {
 	return fmt.Sprintf("%v does not exist", e.Handle)
 }
 
 // IsNotExist returns true if the error was caused by a non-existing file.
 func (b *Backend) IsNotExist(err error) bool {
-	err = errors.Cause(err)
-	_, ok := err.(ErrIsNotExist)
-	return ok
+	var e *notExistError
+	return errors.As(err, &e)
 }
 
 // Load runs fn with a reader that yields the contents of the file at h at the
@@ -284,7 +296,7 @@ func (b *Backend) openReader(ctx context.Context, h restic.Handle, length int, o
 
 	if resp.StatusCode == http.StatusNotFound {
 		_ = resp.Body.Close()
-		return nil, ErrIsNotExist{h}
+		return nil, &notExistError{h}
 	}
 
 	if resp.StatusCode != 200 && resp.StatusCode != 206 {
@@ -329,7 +341,7 @@ func (b *Backend) Stat(ctx context.Context, h restic.Handle) (restic.FileInfo, e
 
 	if resp.StatusCode == http.StatusNotFound {
 		_ = resp.Body.Close()
-		return restic.FileInfo{}, ErrIsNotExist{h}
+		return restic.FileInfo{}, &notExistError{h}
 	}
 
 	if resp.StatusCode != 200 {
@@ -380,7 +392,7 @@ func (b *Backend) Remove(ctx context.Context, h restic.Handle) error {
 
 	if resp.StatusCode == http.StatusNotFound {
 		_ = resp.Body.Close()
-		return ErrIsNotExist{h}
+		return &notExistError{h}
 	}
 
 	if resp.StatusCode != 200 {
