@@ -20,7 +20,7 @@ func findSFTPServerBinary() string {
 	for _, dir := range strings.Split(rtest.TestSFTPPath, ":") {
 		testpath := filepath.Join(dir, "sftp-server")
 		_, err := os.Stat(testpath)
-		if !os.IsNotExist(errors.Cause(err)) {
+		if !errors.Is(err, os.ErrNotExist) {
 			return testpath
 		}
 	}
@@ -42,8 +42,9 @@ func newTestSuite(t testing.TB) *test.Suite {
 			t.Logf("create new backend at %v", dir)
 
 			cfg := sftp.Config{
-				Path:    dir,
-				Command: fmt.Sprintf("%q -e", sftpServer),
+				Path:        dir,
+				Command:     fmt.Sprintf("%q -e", sftpServer),
+				Connections: 5,
 			}
 			return cfg, nil
 		},
