@@ -16,7 +16,7 @@ import (
 )
 
 func TestWriteTar(t *testing.T) {
-	WriteTest(t, WriteTar, checkTar)
+	WriteTest(t, "tar", checkTar)
 }
 
 func checkTar(t *testing.T, testDir string, srcTar *bytes.Buffer) error {
