@@ -8,6 +8,8 @@ import (
 
 	"github.com/restic/restic/internal/repository"
 	"github.com/restic/restic/internal/restic"
+	rtest "github.com/restic/restic/internal/test"
+	"golang.org/x/sync/errgroup"
 )
 
 func randomSize(min, max int) int {
@@ -15,6 +17,9 @@ func randomSize(min, max int) int {
 }
 
 func createRandomBlobs(t testing.TB, repo restic.Repository, blobs int, pData float32) {
+	var wg errgroup.Group
+	repo.StartPackUploader(context.TODO(), &wg)
+
 	for i := 0; i < blobs; i++ {
 		var (
 			tpe    restic.BlobType
@@ -32,7 +37,7 @@ func createRandomBlobs(t testing.TB, repo restic.Repository, blobs int, pData fl
 		buf := make([]byte, length)
 		rand.Read(buf)
 
-		id, exists, err := repo.SaveBlob(context.TODO(), tpe, buf, restic.ID{}, false)
+		id, exists, _, err := repo.SaveBlob(context.TODO(), tpe, buf, restic.ID{}, false)
 		if err != nil {
 			t.Fatalf("SaveFrom() error %v", err)
 		}
@@ -46,6 +51,7 @@ func createRandomBlobs(t testing.TB, repo restic.Repository, blobs int, pData fl
 			if err = repo.Flush(context.Background()); err != nil {
 				t.Fatalf("repo.Flush() returned error %v", err)
 			}
+			repo.StartPackUploader(context.TODO(), &wg)
 		}
 	}
 
@@ -62,7 +68,9 @@ func createRandomWrongBlob(t testing.TB, repo restic.Repository) {
 	// invert first data byte
 	buf[0] ^= 0xff
 
-	_, _, err := repo.SaveBlob(context.TODO(), restic.DataBlob, buf, id, false)
+	var wg errgroup.Group
+	repo.StartPackUploader(context.TODO(), &wg)
+	_, _, _, err := repo.SaveBlob(context.TODO(), restic.DataBlob, buf, id, false)
 	if err != nil {
 		t.Fatalf("SaveFrom() error %v", err)
 	}
@@ -142,7 +150,7 @@ func findPacksForBlobs(t *testing.T, repo restic.Repository, blobs restic.BlobSe
 }
 
 func repack(t *testing.T, repo restic.Repository, packs restic.IDSet, blobs restic.BlobSet) {
-	repackedBlobs, err := repository.Repack(context.TODO(), repo, packs, blobs, nil)
+	repackedBlobs, err := repository.Repack(context.TODO(), repo, repo, packs, blobs, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -155,8 +163,8 @@ func repack(t *testing.T, repo restic.Repository, packs restic.IDSet, blobs rest
 	}
 }
 
-func saveIndex(t *testing.T, repo restic.Repository) {
-	if err := repo.SaveIndex(context.TODO()); err != nil {
+func flush(t *testing.T, repo restic.Repository) {
+	if err := repo.Flush(context.TODO()); err != nil {
 		t.Fatalf("repo.SaveIndex() %v", err)
 	}
 }
@@ -192,9 +200,7 @@ func rebuildIndex(t *testing.T, repo restic.Repository) {
 		t.Fatal(err)
 	}
 
-	_, err = (repo.Index()).(*repository.MasterIndex).
-		Save(context.TODO(), repo, restic.NewIDSet(), nil, nil)
-
+	_, err = repo.Index().Save(context.TODO(), repo, restic.NewIDSet(), nil, nil)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -212,7 +218,11 @@ func reloadIndex(t *testing.T, repo restic.Repository) {
 }
 
 func TestRepack(t *testing.T) {
-	repo, cleanup := repository.TestRepository(t)
+	repository.TestAllVersions(t, testRepack)
+}
+
+func testRepack(t *testing.T, version uint) {
+	repo, cleanup := repository.TestRepositoryWithVersion(t, version)
 	defer cleanup()
 
 	seed := time.Now().UnixNano()
@@ -233,7 +243,7 @@ func TestRepack(t *testing.T) {
 			packsBefore, packsAfter)
 	}
 
-	saveIndex(t, repo)
+	flush(t, repo)
 
 	removeBlobs, keepBlobs := selectBlobs(t, repo, 0.2)
 
@@ -278,8 +288,55 @@ func TestRepack(t *testing.T) {
 	}
 }
 
+func TestRepackCopy(t *testing.T) {
+	repository.TestAllVersions(t, testRepackCopy)
+}
+
+func testRepackCopy(t *testing.T, version uint) {
+	repo, cleanup := repository.TestRepositoryWithVersion(t, version)
+	defer cleanup()
+	dstRepo, dstCleanup := repository.TestRepositoryWithVersion(t, version)
+	defer dstCleanup()
+
+	seed := time.Now().UnixNano()
+	rand.Seed(seed)
+	t.Logf("rand seed is %v", seed)
+
+	createRandomBlobs(t, repo, 100, 0.7)
+	flush(t, repo)
+
+	_, keepBlobs := selectBlobs(t, repo, 0.2)
+	copyPacks := findPacksForBlobs(t, repo, keepBlobs)
+
+	_, err := repository.Repack(context.TODO(), repo, dstRepo, copyPacks, keepBlobs, nil)
+	if err != nil {
+		t.Fatal(err)
+	}
+	rebuildIndex(t, dstRepo)
+	reloadIndex(t, dstRepo)
+
+	idx := dstRepo.Index()
+
+	for h := range keepBlobs {
+		list := idx.Lookup(h)
+		if len(list) == 0 {
+			t.Errorf("unable to find blob %v in repo", h.ID.Str())
+			continue
+		}
+
+		if len(list) != 1 {
+			t.Errorf("expected one pack in the list, got: %v", list)
+			continue
+		}
+	}
+}
+
 func TestRepackWrongBlob(t *testing.T) {
-	repo, cleanup := repository.TestRepository(t)
+	repository.TestAllVersions(t, testRepackWrongBlob)
+}
+
+func testRepackWrongBlob(t *testing.T, version uint) {
+	repo, cleanup := repository.TestRepositoryWithVersion(t, version)
 	defer cleanup()
 
 	seed := time.Now().UnixNano()
@@ -293,9 +350,58 @@ func TestRepackWrongBlob(t *testing.T) {
 	_, keepBlobs := selectBlobs(t, repo, 0)
 	rewritePacks := findPacksForBlobs(t, repo, keepBlobs)
 
-	_, err := repository.Repack(context.TODO(), repo, rewritePacks, keepBlobs, nil)
+	_, err := repository.Repack(context.TODO(), repo, repo, rewritePacks, keepBlobs, nil)
 	if err == nil {
 		t.Fatal("expected repack to fail but got no error")
 	}
 	t.Logf("found expected error: %v", err)
 }
+
+func TestRepackBlobFallback(t *testing.T) {
+	repository.TestAllVersions(t, testRepackBlobFallback)
+}
+
+func testRepackBlobFallback(t *testing.T, version uint) {
+	repo, cleanup := repository.TestRepositoryWithVersion(t, version)
+	defer cleanup()
+
+	seed := time.Now().UnixNano()
+	rand.Seed(seed)
+	t.Logf("rand seed is %v", seed)
+
+	length := randomSize(10*1024, 1024*1024) // 10KiB to 1MiB of data
+	buf := make([]byte, length)
+	rand.Read(buf)
+	id := restic.Hash(buf)
+
+	// corrupted copy
+	modbuf := make([]byte, len(buf))
+	copy(modbuf, buf)
+	// invert first data byte
+	modbuf[0] ^= 0xff
+
+	// create pack with broken copy
+	var wg errgroup.Group
+	repo.StartPackUploader(context.TODO(), &wg)
+	_, _, _, err := repo.SaveBlob(context.TODO(), restic.DataBlob, modbuf, id, false)
+	rtest.OK(t, err)
+	rtest.OK(t, repo.Flush(context.Background()))
+
+	// find pack with damaged blob
+	keepBlobs := restic.NewBlobSet(restic.BlobHandle{Type: restic.DataBlob, ID: id})
+	rewritePacks := findPacksForBlobs(t, repo, keepBlobs)
+
+	// create pack with valid copy
+	repo.StartPackUploader(context.TODO(), &wg)
+	_, _, _, err = repo.SaveBlob(context.TODO(), restic.DataBlob, buf, id, true)
+	rtest.OK(t, err)
+	rtest.OK(t, repo.Flush(context.Background()))
+
+	// repack must fallback to valid copy
+	_, err = repository.Repack(context.TODO(), repo, repo, rewritePacks, keepBlobs, nil)
+	rtest.OK(t, err)
+
+	keepBlobs = restic.NewBlobSet(restic.BlobHandle{Type: restic.DataBlob, ID: id})
+	packs := findPacksForBlobs(t, repo, keepBlobs)
+	rtest.Assert(t, len(packs) == 3, "unexpected number of copies: %v", len(packs))
+}
