@@ -1,3 +1,4 @@
+//go:build !windows
 // +build !windows
 
 package termstatus
@@ -6,7 +7,7 @@ import (
 	"io"
 	"os"
 
-	"golang.org/x/crypto/ssh/terminal"
+	"golang.org/x/term"
 )
 
 // clearCurrentLine removes all characters from the current line and resets the
@@ -23,7 +24,7 @@ func moveCursorUp(wr io.Writer, fd uintptr) func(io.Writer, uintptr, int) {
 // CanUpdateStatus returns true if status lines can be printed, the process
 // output is not redirected to a file or pipe.
 func CanUpdateStatus(fd uintptr) bool {
-	if !terminal.IsTerminal(int(fd)) {
+	if !term.IsTerminal(int(fd)) {
 		return false
 	}
 	term := os.Getenv("TERM")
